/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.require("dojo.io.iframe");
dojo.provide("rmps.importer.ImportClient");

dojo.require("rmps.importer.ImportDefinition");
dojo.require("rmps.importer.Status");
dojo.require("rmps.importer.ProviderStatus");
dojo.require("rmps.importer.CheckEngine");
dojo.require("rmps.core.streams.StreamClient");

dojo.require("rmps.core.client");
dojo.require("jazz.app.context");

dojo.require("rmps.core.RootServices");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.importer", "ImportStatusEditorMessages");

var IMPORT_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "importer");
var STATUS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "status");

var IMPORT_DEFINITION_PATH = IMPORT_PATH + "/definitions/";
var RSA_VERSIONS_PATH = IMPORT_PATH + "/../rsa/systemmodels/rsaVersions";
var IMPORT_REQUEST_PATH = IMPORT_PATH + "/requests/";
var IMPORT_STATUS_PATH = IMPORT_PATH + "/status";
var IMPORT_SAMPLE = IMPORT_PATH + "/sample/";
var IMPORT_PROFILE_PATH = IMPORT_PATH + "/profiles/";
var IMPORT_DEPLOYED_PROFILE_PATH = IMPORT_PATH + "/deployedProfiles/";
var IMPORT_CACHE_DIAGRAMS_REQUEST_PATH = IMPORT_PATH + "/cachediagramsrequest";
var PLUGIN_PROFILES_PATH = IMPORT_PATH + "/profiles-from-plugin";
var IMPORT_PLUGIN_PROFILES_PATH = IMPORT_PATH + "/runtime-profile";

var PROVISION_PATH = IMPORT_PATH + "/../extensionappproxy/dm2ext/provision";

var INSTALL_FEATURE_PATH = PROVISION_PATH + "/install";

var BASELINE_PARAM = "rmps.context=";
var BASELINE_NAME = "baselineName";
var BASELINE_DESCRIPTION = "baselineDescription";

var MESSAGES = dojo.i18n.getLocalization("rmps.importer", "ImportStatusEditorMessages");

rmps.importer.ImportClient = {
	
	checkEngine: function(engineId) {
		var args = {
			url: IMPORT_PATH + "/engines",
			headers: {
				"Accept": "application/rdf+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				for (var uri in response) {
					if (uri.indexOf("_") === 0) {
						var engine = new rmps.importer.CheckEngine();
						engine.parseRdfJson(response[uri]);
						if (engine.id === engineId) {
							return engine;
						}
					}
				}
				return null;
			}
		};
		return jazz.client.xhrGet(args);
	},
	
	checkEngines: function() {
		var args = {
			url: IMPORT_PATH + "/engines",
			headers: {
				"Accept": "application/rdf+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				var engines = [];
				for (var uri in response) {
					if (uri.indexOf("_") === 0) {
						var engine = new rmps.importer.CheckEngine();
						engine.parseRdfJson(response[uri]);
						engines.push(status);
					}
				}
				return engines;
			}
		};
		return jazz.client.xhrGet(args);
	},
	
	getProviderStatus: function(projectId) {
		var args = {
			url: STATUS_PATH + "/import/" + projectId + "?" + BASELINE_PARAM + jazz.app.context.get().baseline,
			headers: {
				"Accept": "application/rdf+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				var statuses = [];
				for (var uri in response) {
					if (uri.indexOf("_") === 0) {
						var status = new rmps.importer.ProviderStatus();
						status.parseRdfJson(projectId, response[uri]);
						
						// Ensure the status is well defined (state, name and web URI)
						if (status.providerName && status.importState && status.importWebUri) { 
							statuses.push(status);
						}
					}
				}
				return statuses;
			}
		};
		return jazz.client.xhrGet(args);
	},
	
	getStatus: function(uri) {
		var args = {
			url: uri + "?" + BASELINE_PARAM + jazz.app.context.get().baseline,
			headers: {
				"Accept": "application/rdf+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				for (var uri in response) {
					var status = new rmps.importer.Status();
					status.parseRdfJson(uri, response[uri]);
					return status;
				}
			}
		};
		return jazz.client.xhrGet(args);
	},
	
	getStatusForGroup: function(groupId) {
		var args = {
			url: IMPORT_STATUS_PATH + "/" + groupId + "?" + BASELINE_PARAM + jazz.app.context.get().baseline,
			headers: {
				"Accept": "application/rdf+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				var statuses = [];
				for (var uri in response) {
					var status = new rmps.importer.Status();
					status.parseRdfJson(uri, response[uri]);
					statuses.push(status);
				}
				return statuses;
			}
		};
		return jazz.client.xhrGet(args);
	},
	
	getStatusForProject: function(projectId, filter) {
		var query = {};
		query.projectId = projectId;
		if (filter) {
			query.filter = filter;
		}
		var args = {
			url: IMPORT_STATUS_PATH + "?" + dojo.objectToQuery(query) + "&" + BASELINE_PARAM + jazz.app.context.get().baseline,
			headers: {
				"Accept": "application/rdf+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				var statuses = [];
				for (var uri in response) {
					var status = new rmps.importer.Status();
					status.parseRdfJson(uri, response[uri]);
					statuses.push(status);
				}
				return statuses;
			}
		};
		return jazz.client.xhrGet(args);
	},
	
	requestImport: function(projectId, reimport, baselineName, baselineDescription) {
		var query = {};
		if (reimport) {
			query.reimport = true;
		}
		
		if (baselineName != null) {
			query.baselineName = baselineName;
			if (baselineDescription != null) {
				query.baselineDescription = baselineDescription;
			}
		}

		var configFile = this._getConfigFile();
		if (configFile != null) {
			query.configFile = configFile;
		}

		var args = {
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			url: IMPORT_REQUEST_PATH + projectId + "?" + dojo.objectToQuery(query) + "&" + BASELINE_PARAM + jazz.app.context.get().baseline,
			hasBody: false,
			load: dojo.hitch(this, function(response, ioArgs) {
				// return the status URI
				return ioArgs.xhr.getResponseHeader("Location");
			})
		};
		return jazz.client.xhrPost(args);
	},
	
	getImportDefinitionUri: function(projectId) {
		return IMPORT_DEFINITION_PATH + projectId;
	},
	
	getImportDefinition: function(uri) {
		var args = {
			url: uri,
			headers: {
				"Accept": "application/rdf+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				var definition = new rmps.importer.ImportDefinition();
				definition.parseRdfJson(response, ioArgs);
				return definition;
			}
		};
		return jazz.client.xhrGet(args);
	},
	
	getImportDefinitionForProject: function(projectId) {
		return this.getImportDefinition(IMPORT_DEFINITION_PATH + projectId + "?" + BASELINE_PARAM + jazz.app.context.get().baseline);
	},
	
	createImportDefinition: function(definition, projectId) {
		var args = {
			headers: {
				"Content-Type": definition.contentType,
				"X-ibm-rmps-internal": "true"
			},
			url: IMPORT_DEFINITION_PATH + projectId  + "?" + BASELINE_PARAM + jazz.app.context.get().baseline,
			hasBody: true,
			postData: definition.toRdfJson(),
			load: dojo.hitch(this, function(response, ioArgs) {
				definition.lastModified = new Date(ioArgs.xhr.getResponseHeader("Last-Modified"));
				definition.eTag = ioArgs.xhr.getResponseHeader("ETag");
				return definition;
			})
		};
		return jazz.client.xhrPost(args);
	},
	
	updateImportDefinition: function(definition) {
		var args = {
			headers: {
				"Content-Type": definition.contentType,
				"If-Match": definition.eTag,
				"X-ibm-rmps-internal": "true"
			},
			url: definition.uri + "?" + BASELINE_PARAM + jazz.app.context.get().baseline,
			hasBody: true,
			putData: definition.toRdfJson(),
			load: dojo.hitch(this, function(response, ioArgs) {
				definition.lastModified = new Date(ioArgs.xhr.getResponseHeader("Last-Modified"));
				definition.eTag = ioArgs.xhr.getResponseHeader("ETag");
				return definition;
			})
		};
		return jazz.client.xhrPut(args);
	},

	deleteImportDefinition: function(/*String|Object*/definition) {
		var uri = dojo.isString(definition) ? definition : definition.uri;
		var args = {
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			url: uri + "?" + BASELINE_PARAM + jazz.app.context.get().baseline,
			load: dojo.hitch(this, function(response, ioArgs) {
				return null;
			})
		};
		return jazz.client.xhrDelete(args);
	},

	deleteStatus: function(statusUri) {
		var args = {
			url: statusUri + "?cancel=false" + "&" + BASELINE_PARAM + jazz.app.context.get().baseline,
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			load: function(response, ioArgs) {
				return null;
			}
		};
		return jazz.client.xhrDelete(args);
	},
	
	cancelImport: function(statusUri) {
		var args = {
			url: statusUri + "?cancel=true" + "&" + BASELINE_PARAM + jazz.app.context.get().baseline,
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			load: function(response, ioArgs) {
				return null;
			}
		};
		return jazz.client.xhrDelete(args);
	},
	
	importSample: function(projectId) {
		// e.g. https://localhost:9444/dm/importer/sample/_duaHUsqHEeCfw5woQqHmFA
		var args = {
			url: IMPORT_SAMPLE + projectId,
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			load: function(response, ioArgs) {
				return ioArgs.xhr.getResponseHeader("Location");
			}
		};
		return jazz.client.xhrPost(args);
	},
	
	requestProfileImport: function(projectId, overwrite, pathMaps, ignoreNsURIs, form) {
		var query = {};
		if (overwrite) {
			query.overwrite = true;
		}
		var configFile = this._getConfigFile();
		if (configFile != null) {
			query.configFile = configFile;
		}
		var theUrl = IMPORT_PROFILE_PATH + projectId + "?" + dojo.objectToQuery(query) + "&" + BASELINE_PARAM + jazz.app.context.get().baseline;
		var args = {
			form: form,
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "html",
			method: "post",
			url: theUrl,
			load: dojo.hitch(this, function(response, ioArgs) {
				var result = response.body.getElementsByTagName("pre");
				return result[0].childNodes[0].data;
			}),
			error: function(error) {
				alert(MESSAGES.error_unknown + "\n" + error);
			} 
		};
		
		//since metamodels are text, we do not need dojo.io.iframe.send
		return dojo.io.iframe.send(args);

	},

	getRSACompatibleVersions: function() {
		var args = {
			url: RSA_VERSIONS_PATH + "?" + BASELINE_PARAM + jazz.app.context.get().baseline,
			headers: {
				"X-ibm-rmps-internal": "true",
				"Accept": "application/json",
			},
			handleAs: "json", //$NON-NLS-1$
			load: dojo.hitch(this, function(response, ioArgs) {
				return response.requiredVersion ? [ response.requiredVersion, response.workspaceSnapshot ] : null;
			})
		};
		return jazz.client.xhrGet(args);
	},

	_getList: function(data) {
		return dojo.filter(data.split("\n"), function (d) { return d.trim() !== ""; });
	},
	
	_hitch: function(f) {
		return dojo.hitch(this, f);
	},
	
	requestDeployedProfileImport: function(projectId, domain, reimport, baselineName, baselineDescription) {
	
		var query = {};
		if (reimport) {
			query.reimport = true;
		}
		
		if (baselineName != null) {
			query.baselineName = baselineName;
			if (baselineDescription != null) {
				query.baselineDescription = baselineDescription;
			}
		}
		
		var args = {
				headers: {
					"Content-Type" : "application/json",
					"X-ibm-rmps-internal": "true"
				},
				url: IMPORT_DEPLOYED_PROFILE_PATH + projectId + "?" + dojo.objectToQuery(query) + "&" + BASELINE_PARAM + jazz.app.context.get().baseline,
				hasBody: true,
				postData : dojo.toJson(domain),
				load: dojo.hitch(this, function(response, ioArgs) {
					// return the status URI
					//return ioArgs.xhr.getResponseHeader("Location");
					return response;
				})
			};
		return jazz.client.xhrPost(args);
	},
	
	cacheDiagramRequest: function() {
		var args = {
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			url: IMPORT_CACHE_DIAGRAMS_REQUEST_PATH,
			hasBody: false,
			load: dojo.hitch(this, function(response, ioArgs) {
				// return the status URI
				return ioArgs.xhr.getResponseHeader("Location");
			})
		};
		return rmps.core.client.xhrPost(args);
	},

	_getConfigFile: function() {
		var url = new dojo._Url(window.location.href);
		return url.query ? dojo.queryToObject(url.query).configFile : null;
	}

};

})();