/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.importer.PluginProfileImportEditor");

dojo.require("dijit._WidgetBase");
dojo.require("dijit._TemplatedMixin");
dojo.require("dijit._WidgetsInTemplateMixin");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.Form");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.importer.ImportClient");
dojo.require("rmps.importer.ImportDefinition");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.importer.ImportStatusEditorInput");
dojo.require("rmps.ui.editor.PageHelper");

dojo.require("rmps.importer.ProfileImportEditor");

dojo.require("rmps.ui.EditableCollapsibleSection");
dojo.require("rmps.ui.pathmaps.PathmapsSection");
dojo.require("rmps.importer.ImportClient");

dojo.require("jazz.ui.Dialog");

dojo.require("dojox.xml.DomParser");
dojo.require("rmpw.common.util.ErrorUtil");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.importer", "PluginProfileImportEditorMessages");
var messages = dojo.i18n.getLocalization("rmps.importer", "PluginProfileImportEditorMessages");
var projectId;

dojo.declare("rmps.importer.PluginProfileImportEditor", rmps.app.ui.internal.editors.BaseEditor, {
	"class": "rmps-plugin-profile-importer",
	
	//_definition: null,
	_overwrite: true,
	
	_requestButton: null,
	_form: null,
	_content: null,

	setForm: function(form) {
		this._form = form; 
	},

	postCreate: function() {
		this.inherited(arguments);
		var deferred = rmps.importer.ImportClient.getRSACompatibleVersions();
		deferred.addCallback(this, function(versions) {
			if (versions) {
				this._requestButton.domNode.style.display = "";
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
					summary: dojo.string.substitute(messages.editorPreImportWarning, versions),
					closable: true
				});
			} else {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.errorNoDependencyOnRSADomains,
					closable: true
				});
			}
		});
		deferred.addErrback(this, function(error) {
			this.showMessage({type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				 		      summary: messages.errorOnCheckingRSARequiredVersion,
						      closable: true});
		});
	},

	createContentWidget: function() {
		//this._definition = new rmps.importer.ImportDefinition();
		this._loadCookie();
		return this._content = new rmps.importer.PluginProfileImportEditor._Content({_editor: this});
	},
	
	addButtons: function(toolbar) {
		this._requestButton = new rmps.ui.SimpleButton({
			label: messages.editorInputButton,
			onClick: dojo.hitch(this, this.requestImport)
		});
		this._requestButton.domNode.style.display = "none";
		toolbar.addWidget(this._requestButton);
	},
	
	_doWarning: function(warning) {
		this.contentWidget._errorsAndWarningsWidget.makeVisible(warning);
	},
	
	_hideWarning: function(warning) {
		this.contentWidget._errorsAndWarningsWidget.makeInvisible();
	},
	
	_disableRedo: function(warning) {
		this.contentWidget._errorsAndWarningsWidget.disableRedo();
	},	
	
	requestImport: function() {
	    this._requestImport(this);
	},
	
	_requestImport: function(editor, duplicates) {
		if (!editor) {
			editor = this;
		}
		projectId = jazz.app.context.get().itemId;
		
		var deferred = rmps.importer.ImportClient.requestDeployedProfileImport(projectId, this._content.domainData(), true, this._content.snapshot(), this._content.snapshotDescription());
		deferred.addCallback(editor, function(result) {
			
			if (result === "ok") {
				editor._hideWarning();
				this._refreshStatus();
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.OK,
					summary: dojo.string.substitute(
						messages.importRequestSuccessful,
						[rmps.core.util.IRmpsConstants.ADMIN_IMPORT_STATUS]),
					closable: true
				});
			} else if (result === "warning") {
				editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
					summary: messages.editorImportWarning,
					closable: true
				});
				editor._doWarning(result);
			} else {
				editor._hideWarning();
				editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.editorImportError,
					details: [ {message: messages['editorImportError.explanation'] + "<br>" + messages['editorImportError.useraction']}],
					closable: true
				});
			}
			
		});
		deferred.addErrback(editor, function(error) {
			editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.editorImportSeriousError,
				details: [ {message:dojo.string.substitute(messages['editorImportSeriousError.explanation'], [error.status]) + "<br>" + messages['editorImportSeriousError.useraction']}],
				closable: true
			});
		});
	},
	
	_showMessage: function(editor, result) {
		editor.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.OK,
			summary: result,
			closable: true
		});
	},
	
	_showError: function(editor, error) {
		var rdf = dojox.xml.DomParser.parse(error.detailedMessage);
		editor.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: messages.error,
			details: [{message: rdf.getElementsByTagName("err:errorMessage")[0].childNodes[0].nodeValue}],
			closable: true
		});
	},
	
	_showProgress: function(editor, update) {
		editor.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: messages.importInProgress + update,
			closable: false
		});
	},
	
	_refreshStatus: function() {
		var helper = rmps.ui.editor.PageHelper.getInstance();
		var statusEditor = helper.getEditor(
				new rmps.importer.ImportStatusEditorInput({projectId: projectId}));
		if (statusEditor) {
			statusEditor.refresh();
		}
	},

	_loadCookie: function() {
		var value = dojo.cookie(this.declaredClass);
		if (value) {
			var cookie = dojo.fromJson(value);
			this._overwrite = !!cookie.overwrite;
		}
	},
	
	_saveCookie: function() {
		var value = dojo.cookie(this.declaredClass);
		var cookie;
		if (value) {
			cookie = dojo.fromJson(value);
		} else {
			cookie = {};
		}
		cookie.overwrite = this._overwrite;
		dojo.cookie(this.declaredClass, dojo.toJson(cookie), {expires: 90});
	},
	
	resize: function() {
		//override: do not resize
	}
});

dojo.declare("rmps.importer.PluginProfileImportEditor._Content", [dijit._WidgetBase, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], {
	templateString: dojo.cache("rmps.importer", "templates/PluginProfileImportEditor_Content.html"),
	_editor: null,
	messages: messages,

	_init: function() {
		this._definition = this._editor._definition;

		this._errorsAndWarningsWidget = new rmps.importer.ProfileImportEditor._ErrorsAndWarningsContent({_editor :this._editor});
		this._editor.containerNode.appendChild(this._errorsAndWarningsWidget.domNode);
		this._errorsAndWarningsWidget.startup();

		this._editor.containerNode.appendChild(this.domNode);
	},
	
	postCreate: function() {
		this.inherited(arguments);
		this._init();
	},

	_validateInput: function() {
		return dojo.every(["_domainName", "_domainId", "_domainUri", "_profileUri"], dojo.hitch(this, function (name) {
			return !!this[name].value;
		}));		
	},
	
	_onChange: function() {
		this._updateCb && this._updateCb(this._validateInput());
	},
	
	domainData: function() {
		return {name: this._domainName.value, id: this._domainId.value, uri: this._domainUri.value, profiles: [this._rsaProfileUri.value]};
	},
	
	snapshot: function() {
		return this._snapshot.value || null;
	},

	snapshotDescription: function() {
		return this._snapshotDesc.value || null;
	}
});

})();
